@@ -1,4 +1,4 @@
-/* Copyright (C) 2010-2024 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2010-2025 by Arm Limited. All rights reserved. */
 
 #include "CapturedXML.h"
 
@@ -11,11 +11,13 @@
 #include "Logging.h"
 #include "OlyUtility.h"
 #include "PrimarySourceProvider.h"
+#include "ProductVersion.h"
 #include "ProtocolVersion.h"
 #include "SessionData.h"
 #include "lib/FsEntry.h"
 #include "lib/Span.h"
 #include "lib/String.h"
+#include "libGPUInfo/source/libgpuinfo.hpp"
 #include "xml/MxmlUtils.h"
 
 #include <algorithm>
@@ -26,6 +28,7 @@
 #include <ctime>
 #include <map>
 #include <memory>
+#include <regex>
 #include <set>
 #include <string>
 #include <vector>
@@ -95,7 +98,7 @@ static std::string modeAsString(const ConstantMode mode)
 }
 
 /** Generate the xml tree for capture.xml */
-//NOLINTNEXTLINE(readability-function-cognitive-complexity)
+// NOLINTNEXTLINE(readability-function-cognitive-complexity)
 static mxml_node_t * getTree(bool includeTime,
                              lib::Span<const CapturedSpe> spes,
                              const std::vector<TemplateConfiguration> & templateConfiguration,
@@ -111,6 +114,8 @@ static mxml_node_t * getTree(bool includeTime,
                                                           : "none");
     mxmlElementSetAttr(captured, "type", primarySourceProvider.getCaptureXmlTypeValue());
     mxmlElementSetAttrf(captured, "protocol", "%d", PROTOCOL_VERSION);
+    mxmlElementSetAttrf(captured, "product", "%d", PRODUCT_VERSION);
+    mxmlElementSetAttrf(captured, "product_tag", "%s", PRODUCT_VERSION_BRANCH_NAME);
     if (includeTime) {                    // Send the following only after the capture is complete
         if (time(nullptr) > 1267000000) { // If the time is reasonable (after Feb 23, 2010)
             mxmlElementSetAttrf(captured,
@@ -140,13 +145,25 @@ static mxml_node_t * getTree(bool includeTime,
 
     const auto & cpuInfo = primarySourceProvider.getCpuInfo();
     mxmlElementSetAttr(target, "name", cpuInfo.getModelName());
+
+    // Get the instance at id 0, as in multi-gpu case gpus are homogenous.
+    const auto gpuInfoInstance = libarmgpuinfo::instance::create();
+
+    if (gpuInfoInstance != nullptr) {
+        const auto * const gpuPublicName = gpuInfoInstance->get_info().gpu_name;
+
+        if (!maliGpuIds.empty() && gpuPublicName != nullptr) {
+            mxmlElementSetAttr(target, "gpu_public_name", gpuPublicName);
+        }
+    }
+
     const auto midrs = cpuInfo.getMidrs();
     mxmlElementSetAttrf(target, "cores", "%zu", midrs.size());
-    //GPU cores
+    // GPU cores
     mxmlElementSetAttrf(target, "gpu_cores", "%zu", maliGpuIds.size());
-    //gatord src md5
+    // gatord src md5
     mxmlElementSetAttrf(target, "gatord_src_md5sum", "%s", gSrcMd5);
-    //gatord build commit id
+    // gatord build commit id
     mxmlElementSetAttrf(target, "gatord_build_id", "%s", gBuildId);
 
     assert(midrs.size() > 0); // gatord should've died earlier if there were no cpus
