@@ -1,10 +1,13 @@
-/* Copyright (C) 2010-2023 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2010-2024 by Arm Limited. All rights reserved. */
 
 #ifndef __ISENDER_H__
 #define __ISENDER_H__
 
 #include "lib/Span.h"
 
+#include <cstdint>
+#include <string>
+
 enum class ResponseType : char {
     /// Special value used by ISender meaning do not frame the response.
     RAW = 0,
@@ -25,14 +28,22 @@ public:
     /**
      * @param dataParts must be a complete response unless type is RAW
      */
-    virtual void writeDataParts(lib::Span<const lib::Span<const char, int>> dataParts,
+    virtual void writeDataParts(lib::Span<const lib::Span<const uint8_t, int>> dataParts,
                                 ResponseType type,
                                 bool ignoreLockErrors = false) = 0;
 
-    void writeData(const char * data, int length, ResponseType type, bool ignoreLockErrors = false)
+    void writeData(const uint8_t * data, int length, ResponseType type, bool ignoreLockErrors = false)
     {
-        lib::Span<const char, int> dataSpan = {data, length};
-        writeDataParts(lib::Span<const lib::Span<const char, int>> {&dataSpan, 1}, type, ignoreLockErrors);
+        lib::Span<const uint8_t, int> dataSpan = {data, length};
+        writeDataParts(lib::Span<const lib::Span<const uint8_t, int>> {&dataSpan, 1}, type, ignoreLockErrors);
+    }
+
+    void writeData(const std::string & string_data, ResponseType type, bool ignoreLockErrors = false)
+    {
+        writeData(reinterpret_cast<const std::uint8_t *>(string_data.data()),
+                  string_data.length(),
+                  type,
+                  ignoreLockErrors);
     }
 
     virtual ~ISender() = default;
