@@ -3,8 +3,6 @@
 #ifndef OLY_UTILITY_H
 #define OLY_UTILITY_H
 
-#include <cstddef>
-
 #ifdef WIN32
 #define PATH_SEPARATOR '\\'
 #if !defined(_MSC_VER) || _MSC_VER < 1900
@@ -14,11 +12,13 @@
 #define PATH_SEPARATOR '/'
 #endif
 
+enum class OlyBase : int { Auto = 0, Binary, Decimal = 10, Hexadecimal = 16 };
+
 bool stringToBool(const char * string, bool defValue);
 void stringToLower(char * string);
-bool stringToLongLong(long long * value, const char * str, int base);
-bool stringToLong(long * value, const char * str, int base);
-bool stringToInt(int * value, const char * str, int base);
+bool stringToLongLong(long long * value, const char * str, OlyBase base = OlyBase::Auto);
+bool stringToLong(long * value, const char * str, OlyBase base = OlyBase::Auto);
+bool stringToInt(int * value, const char * str, OlyBase base = OlyBase::Auto);
 int getApplicationFullPath(char * path, int sizeOfPath);
 char * readFromDisk(const char * file, unsigned int * size = nullptr, bool appendNull = true);
 int writeToDisk(const char * path, const char * data);
