@@ -1,4 +1,4 @@
-/* Copyright (C) 2017-2020 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2017-2024 by Arm Limited. All rights reserved. */
 
 #ifndef INCLUDE_LIB_TIMESTAMPSOURCE_H
 #define INCLUDE_LIB_TIMESTAMPSOURCE_H
@@ -13,9 +13,9 @@ namespace lib {
     public:
         TimestampSource(clockid_t id);
 
-        unsigned long long getBaseTimestampNS() const;
-        unsigned long long getTimestampNS() const;
-        unsigned long long getAbsTimestampNS() const;
+        [[nodiscard]] unsigned long long getBaseTimestampNS() const;
+        [[nodiscard]] unsigned long long getTimestampNS() const;
+        [[nodiscard]] unsigned long long getAbsTimestampNS() const;
 
     private:
         unsigned long long base;
