@@ -1,4 +1,4 @@
-/* Copyright (C) 2013-2024 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2013-2025 by Arm Limited. All rights reserved. */
 
 #include "linux/perf/PerfDriverConfiguration.h"
 
@@ -9,6 +9,8 @@
 #include "capture/Environment.h"
 #include "k/perf_event.h"
 #include "lib/AutoClosingFd.h"
+#include "lib/CpuIdSet.h"
+#include "lib/Error.h"
 #include "lib/FileDescriptor.h"
 #include "lib/Format.h"
 #include "lib/FsEntry.h"
@@ -30,7 +32,6 @@
 #include <cstdint>
 #include <cstdlib>
 #include <cstring>
-#include <functional>
 #include <memory>
 #include <optional>
 #include <set>
@@ -38,6 +39,7 @@
 #include <string_view>
 #include <unordered_map>
 #include <utility>
+#include <vector>
 
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
@@ -47,6 +49,10 @@
 #include <sched.h>
 #endif
 
+#include "setup_warnings.h"
+
+#include <sstream>
+
 #include <sys/utsname.h>
 #include <unistd.h>
 
@@ -63,11 +69,10 @@ namespace {
         const char * const command[] = {"getprop", securityPerfHardenPropString.data(), nullptr};
         const lib::PopenResult getprop = lib::popen(command);
         if (getprop.pid < 0) {
-            //NOLINTNEXTLINE(concurrency-mt-unsafe)
             LOG_DEBUG("lib::popen(%s %s) failed: %s. Probably not android",
                       command[0],
                       command[1],
-                      std::strerror(-getprop.pid));
+                      lib::strerror(-getprop.pid));
             return false;
         }
 
@@ -84,21 +89,19 @@ namespace {
         const lib::PopenResult setPropResult = lib::popen(command);
         //setprop not found, probably not Android.
         if (setPropResult.pid == -ENOENT) {
-            //NOLINTNEXTLINE(concurrency-mt-unsafe)
             LOG_DEBUG("lib::popen(%s %s %s) failed: %s",
                       command[0],
                       command[1],
                       command[2],
-                      std::strerror(-setPropResult.pid));
+                      lib::strerror(-setPropResult.pid));
             return;
         }
         if (setPropResult.pid < 0) {
-            //NOLINTNEXTLINE(concurrency-mt-unsafe)
             LOG_ERROR("lib::popen(%s %s %s) failed: %s",
                       command[0],
                       command[1],
                       command[2],
-                      std::strerror(-setPropResult.pid));
+                      lib::strerror(-setPropResult.pid));
             return;
         }
 
@@ -255,9 +258,7 @@ namespace {
             }
         }
         else {
-            auto const e = errno;
-            //NOLINTNEXTLINE(concurrency-mt-unsafe)
-            LOG_DEBUG("Failed to read data from counters due to res=%zd, errno=%d (%s)", res, e, std::strerror(e));
+            LOG_DEBUG("Failed to read data from counters due to res=%zd, errno=%d (%s)", res, errno, lib::strerror());
         }
 
         return result;
@@ -268,31 +269,24 @@ namespace {
         constexpr std::size_t max_possible_events = 32;
         constexpr unsigned affine_loop_count = 5;
 
-        auto const cpu_set_size = CPU_ALLOC_SIZE(cpuNo + 1);
-
         std::vector<lib::AutoClosingFd> event_fds {};
 
         LOG_DEBUG("Determining max events for #%zu", cpuNo);
 
-        // Set the affinity to just that CPU so that it is online, and so that there are some events generated by perf
-        std::unique_ptr<cpu_set_t, std::function<void(cpu_set_t *)>> cpuset {CPU_ALLOC(cpuNo + 1),
-                                                                             [](cpu_set_t * ptr) { CPU_FREE(ptr); }};
-
-        CPU_ZERO_S(cpu_set_size, cpuset.get());
-        CPU_SET_S(cpuNo, cpu_set_size, cpuset.get());
+        lib::CpuIdSet cpuset(cpuNo + 1);
+        cpuset.add(cpuNo);
 
         // try and set affinity to ensure cpuNo is doing some work (and won't read 0 events)
         // Note: This needs to be restored elsewhere.
         bool affinitySucceeded = false;
         for (unsigned count = 0; count < affine_loop_count && !affinitySucceeded; ++count) {
-            if (sched_setaffinity(0, cpu_set_size, cpuset.get()) == 0) {
+            if (lib::sched_setaffinity(0, cpuset) == 0) {
                 affinitySucceeded = true;
             }
         }
 
         if (!affinitySucceeded) {
-            //NOLINTNEXTLINE(concurrency-mt-unsafe)
-            LOG_WARNING("Error calling sched_setaffinity on %zu: %d (%s)", cpuNo, errno, strerror(errno));
+            LOG_WARNING("Error calling sched_setaffinity on %zu: %d (%s)", cpuNo, errno, lib::strerror());
         }
 
         // make sure we are definitely running on the CPU...
@@ -330,9 +324,7 @@ namespace {
                 }
             }
             else {
-                auto const e = errno;
-                //NOLINTNEXTLINE(concurrency-mt-unsafe)
-                LOG_DEBUG("Failed to open test event# %zu due to %d (%s)", n, e, std::strerror(e));
+                LOG_DEBUG("Failed to open test event# %zu due to %d (%s)", n, errno, lib::strerror());
                 break;
             }
         }
@@ -354,23 +346,14 @@ namespace {
         }
 
         // record the current thread affinity so it can be restored later
-        auto const cpu_set_size = CPU_ALLOC_SIZE(midrs.size());
+        lib::CpuIdSet original_cpuset(midrs.size());
+        if (lib::sched_getaffinity(0, original_cpuset) < 0) {
+            LOG_DEBUG("Error calling sched_getaffinity to get current mask: %d (%s)", errno, lib::strerror());
 
-        std::unique_ptr<cpu_set_t, std::function<void(cpu_set_t *)>> original_cpuset {
-            CPU_ALLOC(midrs.size()),
-            [](cpu_set_t * ptr) { CPU_FREE(ptr); }};
-
-        CPU_ZERO_S(cpu_set_size, original_cpuset.get());
-
-        if (sched_getaffinity(0, cpu_set_size, original_cpuset.get()) < 0) {
-            auto const e = errno;
-            //NOLINTNEXTLINE(concurrency-mt-unsafe)
-            LOG_DEBUG("Error calling sched_getaffinity to get current mask: %d (%s)", e, strerror(e));
-
-            CPU_ZERO_S(cpu_set_size, original_cpuset.get());
+            original_cpuset.clear();
 
             for (std::size_t cpuNo = 0; cpuNo < midrs.size(); ++cpuNo) {
-                CPU_SET_S(cpuNo, cpu_set_size, original_cpuset.get());
+                original_cpuset.add(cpuNo);
             }
         }
 
@@ -398,20 +381,33 @@ namespace {
             // try and set affinity
             bool affinitySucceeded = false;
             for (unsigned count = 0; count < affine_loop_count && !affinitySucceeded; ++count) {
-                if (sched_setaffinity(0, cpu_set_size, original_cpuset.get()) == 0) {
+                if (lib::sched_setaffinity(0, original_cpuset) == 0) {
                     affinitySucceeded = true;
                 }
             }
 
             if (!affinitySucceeded) {
-                auto const e = errno;
-                //NOLINTNEXTLINE(concurrency-mt-unsafe)
-                LOG_WARNING("Error calling sched_setaffinity to restore all: %d (%s)", e, strerror(e));
+                LOG_WARNING("Error calling sched_setaffinity to restore all: %d (%s)", errno, lib::strerror());
             }
         }
 
         return max_event_count_by_cpuid;
     }
+
+    void create_perf_event_paranoid_error(setup_warnings_t & setup_warnings,
+                                          int current_paraniod_value,
+                                          bool wants_system_wide)
+    {
+        const auto needed_paranoid_value = wants_system_wide ? -1 : 2;
+        std::stringstream str;
+        str << "The perf security settings will prevent collection of profiling data. The value of "
+               "/proc/sys/kernel/perf_event_paranoid is "
+            << current_paraniod_value
+            << ". Run the following command on the target to enable data collection: "
+               " `echo "
+            << needed_paranoid_value << " > /proc/sys/kernel/perf_event_paranoid`";
+        setup_warnings.add_error(str.str());
+    }
 }
 
 void logCpuNotFound()
@@ -428,7 +424,8 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                                                                          const char * tracefsEventsPath,
                                                                          lib::Span<const cpu_utils::midr_t> midrs,
                                                                          const default_identifiers_t & smmu_identifiers,
-                                                                         const PmuXML & pmuXml)
+                                                                         const PmuXML & pmuXml,
+                                                                         setup_warnings_t & setup_warnings)
 {
     struct utsname utsname;
     if (lib::uname(&utsname) != 0) {
@@ -442,6 +439,7 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
 
     // Check the kernel version
     auto const kernelVersion = lib::parseLinuxVersion(utsname);
+    setup_warnings.kernel_version = kernelVersion;
 
     const bool hasArmv7PmuDriver = beginsWith(utsname.machine, "armv7")
                                 || FsEntry::create("/sys/bus/event_source/devices").hasChildWithNamePrefix("armv7");
@@ -461,10 +459,12 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
         static const char * error_message = "Unsupported kernel version\nPlease upgrade to 3.4 or later";
         LOG_SETUP("%s", error_message);
         LOG_ERROR("%s", error_message);
+        setup_warnings.add_error("The target's kernel is too old. Please upgrade to kernel version 3.4 or later.");
         return nullptr;
     }
 
     const auto os_type = capture::detectOs();
+    setup_warnings.os_type = os_type;
     const bool is_android = (os_type == capture::OsType::Android);
 
     const bool isRoot = (lib::geteuid() == 0);
@@ -478,6 +478,9 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                   "Try 'setprop %s 0' as the shell or root user.",
                   securityPerfHardenPropString.data(),
                   securityPerfHardenPropString.data());
+        setup_warnings.add_error("The Android security settings will prevent collection of perf data. "
+                                 "Please run the following command on the device as the shell or root user: "
+                                 "`setprop security.perf_harden 0`");
         return nullptr;
     }
 
@@ -488,6 +491,8 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                       "Is CONFIG_PERF_EVENTS enabled?");
             LOG_ERROR("perf_event_paranoid not accessible\n"
                       "Is CONFIG_PERF_EVENTS enabled?");
+            setup_warnings.add_error("The /proc/sys/kernel/perf_event_paranoid file could not be read. "
+                                     "Please check that perf support is enabled for this kernel.");
             return nullptr;
         }
 #if defined(CONFIG_ASSUME_PERF_HIGH_PARANOIA) && CONFIG_ASSUME_PERF_HIGH_PARANOIA
@@ -505,6 +510,8 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
     const bool can_collect_kernel_data = isRoot || (perf_event_paranoid <= 1);
     const bool can_collect_any_data = isRoot || perf_event_paranoid <= 2;
 
+    bool has_errors = false;
+
     if (!can_collect_any_data) {
         // This is only actually true if the kernel has the grsecurity PERF_HARDEN patch
         // but we assume no-one would ever set perf_event_paranoid > 2 without it.
@@ -512,10 +519,11 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
         LOG_ERROR("perf_event_open: perf_event_paranoid > 2 is not supported for non-root.\n"
                   "To use it try (as root):\n"
                   "  echo 2 > /proc/sys/kernel/perf_event_paranoid");
-        return nullptr;
-    }
 
-    if (systemWide && !can_collect_system_wide_data) {
+        create_perf_event_paranoid_error(setup_warnings, perf_event_paranoid, systemWide);
+        has_errors = true;
+    }
+    else if (systemWide && !can_collect_system_wide_data) {
         LOG_SETUP("System wide tracing\nperf_event_paranoid > 0 is not supported for system-wide non-root");
         LOG_ERROR("perf_event_open: perf_event_paranoid > 0 is not supported for system-wide non-root.\n"
                   "To use it\n"
@@ -524,7 +532,8 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                   " * or make sure '/proc/sys/kernel/perf_event_paranoid' is set to -1.\n"
                   "   Try (as root):\n"
                   "    - echo -1 > /proc/sys/kernel/perf_event_paranoid");
-        return nullptr;
+        create_perf_event_paranoid_error(setup_warnings, perf_event_paranoid, systemWide);
+        has_errors = true;
     }
 
     const bool can_access_tracepoints = (lib::access(tracefsEventsPath, R_OK) == 0);
@@ -547,6 +556,9 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                       " * or make sure '/proc/sys/kernel/perf_event_paranoid' is set to -1.\n"
                       "   Try (as root):\n"
                       "    - echo -1 > /proc/sys/kernel/perf_event_paranoid");
+
+            create_perf_event_paranoid_error(setup_warnings, perf_event_paranoid, systemWide);
+            has_errors = true;
         }
         else {
             if (isRoot) {
@@ -556,6 +568,12 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                           "Try:\n"
                           " - mount -t debugfs none /sys/kernel/debug",
                           tracefsEventsPath);
+                std::stringstream msg("Profiling in system-wide mode requires access to '");
+                msg << tracefsEventsPath
+                    << "' but this is not accessible. Check that debugfs is mounted. "
+                       "You can run the following command on the target to mount it: 'mount -t debugfs none "
+                       "/sys/kernel/debug'";
+                setup_warnings.add_warning(msg.str());
             }
             else {
                 LOG_SETUP("%s does not exist\nIs CONFIG_TRACING and CONFIG_CONTEXT_SWITCH_TRACER enabled?",
@@ -568,8 +586,21 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                           "    - mount -o remount,mode=755 /sys/kernel/debug\n"
                           "    - mount -o remount,mode=755 /sys/kernel/debug/tracing",
                           tracefsEventsPath);
+                std::stringstream msg("Profiling in system-wide mode requires access to '");
+                msg << tracefsEventsPath
+                    << "' but this is not accessible with the current user. "
+                       "Check that debugfs is mounted with the correct read/write permissions. "
+                       "You can run the following commands on the target to remount it: "
+                       "'mount -o remount,mode=755 /sys/kernel/debug' and 'mount -o remount,mode=755 "
+                       "/sys/kernel/debug/tracing'";
+                setup_warnings.add_warning(msg.str());
             }
         }
+        has_errors = true;
+    }
+
+    // everything after here requires perf so, if we've already got errors, there's no point continuing.
+    if (has_errors) {
         return nullptr;
     }
 
@@ -584,6 +615,7 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
     configuration->config.has_sample_identifier = (kernelVersion >= KERNEL_VERSION(3U, 12U, 0U));
     configuration->config.has_attr_comm_exec = (kernelVersion >= KERNEL_VERSION(3U, 16U, 0U));
     configuration->config.has_attr_mmap2 = (kernelVersion >= KERNEL_VERSION(3U, 16U, 0U));
+    configuration->config.has_attr_build_id = (kernelVersion >= KERNEL_VERSION(5U, 12U, 0U));
     configuration->config.has_attr_clockid_support = (kernelVersion >= KERNEL_VERSION(4U, 1U, 0U));
     configuration->config.has_attr_context_switch = (kernelVersion >= KERNEL_VERSION(4U, 3U, 0U));
     configuration->config.has_ioctl_read_id = (kernelVersion >= KERNEL_VERSION(3U, 12U, 0U));
@@ -625,12 +657,18 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
         if (fd >= 0) {
             LOG_DEBUG("Detected support for alternative sample period features");
             configuration->config.supports_strobing_core = true;
+            setup_warnings.supports_counter_strobing = tri_bool_t::yes;
+
             close(fd);
         }
         else {
             auto const e = errno;
             //NOLINTNEXTLINE(concurrency-mt-unsafe)
             LOG_DEBUG("No support for alternative sample period features, error was %d (%s)", e, std::strerror(e));
+            setup_warnings.supports_counter_strobing = tri_bool_t::no;
+            setup_warnings.add_warning("The target does not support perf counter strobing. Metrics collection "
+                                       "may result in high CPU usage. To resolve this warning, recompile your kernel "
+                                       "with the counter strobing patch applied.");
         }
     }
 
@@ -655,17 +693,23 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
         if (fd >= 0) {
             LOG_DEBUG("Detected support for inheritable counter groups");
             configuration->config.supports_inherit_sample_read = true;
+            setup_warnings.supports_event_inherit = tri_bool_t::yes;
             close(fd);
         }
         else {
             auto const e = errno;
             //NOLINTNEXTLINE(concurrency-mt-unsafe)
             LOG_DEBUG("No support for inheritable counter groups, error was %d (%s)", e, std::strerror(e));
+            setup_warnings.supports_event_inherit = tri_bool_t::no;
+            setup_warnings.add_warning("The target does not support inheritable counter groups. This can result "
+                                       "in creation of large numbers of file descriptors which might cause the capture "
+                                       "to fail.");
         }
     }
 
     // detect max number of events per PMU
     auto const max_event_count_by_cpuid = calculate_max_event_count_by_cpuid(midrs);
+    setup_warnings.number_of_counters_by_cpu = max_event_count_by_cpuid;
 
     // detect the PMUs
     std::set<const GatorCpu *> cpusDetectedViaSysFs;
@@ -736,10 +780,10 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                 if (lib::readIntFromFile(typePath.c_str(), type) == 0) {
                     configuration->config.supports_strobing_patches |= pmu_supports_strobing_patches(*dirent);
 
-                    const std::set<int> cpuNumbers = lib::readCpuMaskFromFile(maskPath.c_str());
+                    const auto cpuNumbers = lib::readCpuMaskFromFile(maskPath.c_str());
                     if (!cpuNumbers.empty()) {
                         cpu_utils::cpuid_t cpuIdForType;
-                        for (int cpuNumber : cpuNumbers) {
+                        for (const int cpuNumber : cpuNumbers) {
                             // track generic pmu's type to the cpuId associated with it.
                             // if multiple different cpuIds are associated, then use -1 as cannot map to a unique pmu.
                             const auto cpuIdForCpu = midrs[cpuNumber].to_cpuid();
@@ -782,8 +826,8 @@ std::unique_ptr<PerfDriverConfiguration> PerfDriverConfiguration::detect(Capture
                 const std::string typePath(lib::Format() << PERF_DEVICES << "/" << name << "/type");
                 const std::string maskPath(lib::Format() << PERF_DEVICES << "/" << name << "/cpumask");
                 if (lib::readIntFromFile(typePath.c_str(), type) == 0) {
-                    const std::set<int> cpuNumbers = lib::readCpuMaskFromFile(maskPath.c_str());
-                    for (int cpuNumber : cpuNumbers) {
+                    const auto cpuNumbers = lib::readCpuMaskFromFile(maskPath.c_str());
+                    for (const int cpuNumber : cpuNumbers) {
                         LOG_DEBUG("    ... using SPE pmu type %d for cpu %d", type, cpuNumber);
                         configuration->cpuNumberToSpeType[cpuNumber] = type;
                     }
